from chemistry.conversions import Conversions
from chemistry.formulas import Formulas
from chemistry.periodic import *

# constants
c = 299792458
a = 6.0221408 * (10 ** 23)
e = -1 * 1.60217663 * (10 ** (-1 * 19))
R = 8.314